# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Standalone- Interface for an KVMHCRunProcess.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from optparse import OptionParser
import re
from multiprocessing import Queue
import logging
import time

from mapwoc.lib.hc import HC
from mapwoc.lib.hccontrol import HCControl

from mapwoc.nodectrl.kvmhcrunprocess import KVMHCRunProcess
from  mapwoc.nodectrl.hcinteractionscript import eventmap_from_string


parser = OptionParser()
parser.add_option("-i", "--image", dest="image", help="path to image file")
parser.add_option("-t", "--timeout", dest="timeout", default=60, type="int",
                  help="timeout ('active window') in seconds (default=60sec)")
parser.add_option("", "--vnc", dest="vnc", default=30, type="int",
                  help="VNC Port to start at (negative for nongraphic).")
parser.add_option("-s", "--snapshot", dest="snapshot", help="snapshot to load from image")
parser.add_option("-e", "--events", dest="events", help="When to send "
                                    "alt-home key presses to the vm, e.g. 5r10r30r")
parser.add_option("-l", "--loop", dest="loop", action="store_true",
                  help="Loop forever (defunc).")
parser.add_option("-v", "--verbose", dest="verbose", action="store_true",
                  help="Be verbose.")
parser.add_option("-p", "--poison", dest="poison", action="store_true",
                  help="Give process poison after 5 seconds.")
parser.add_option("-f", "--hcfile", dest="hcfile",
                  help="HCControl file to use"
                       " (this option conflicts --snapshot and --image.")
(options, args) = parser.parse_args()

hc_explicit = False
hc_file = False

if options.image is not None and options.snapshot is not None:
    hc_explicit = True

if bool(options.image) != bool(options.snapshot):
    print "When the image option is specified, snapshot has to be specified, too."
    exit (1)

if options.hcfile is not None:
    hc_file = True

if not hc_file and not hc_explicit:
    print "Please provide at least the snapshot and image or hcfile 'options'."
    exit (1)
elif hc_file and hc_explicit:
    print "The options snapshot, image and hcfile are conflicting."
    exit (1)

if options.loop:
    print "Loop feature is disabled/not implemented."

if options.verbose:
    nodelogger = logging.getLogger('node')
    nodelogger.addHandler(logging.StreamHandler())
    nodelogger.setLevel(logging.DEBUG)

hc = HCControl(filepath=options.hcfile)
if hc_explicit:
    hc.image_file = options.image
    hc.image_snapshot = options.snapshot
    hc.hctype = 'kvm'
    # Use somewhat sensible default values
    hc.dns_ip = '10.0.0.3'
    hc.gateway_ip = '10.0.0.2'
    hc.own_ip = '192.168.1.1'
    hg.homepage = 'http://10.0.0.4/hcsid'

eventmap = eventmap_from_string(options.events, hc)

try:
    # loop is not a feature of KVMHCRunProcess anymore, implement it here
    results = Queue()
    commands = Queue()
    kvmp = KVMHCRunProcess(hc, eventmap, options.timeout, commands, results, './')
    if options.vnc > 0:
        kvmp.vnc_port = options.vnc

    print "Starting KVM Process."
    kvmp.start()
    if options.poison:
        time.sleep(5)
        commands.put("die")

    kvmp.join()

    print "Joined the KVM Process."
    try:
        if results.empty():
            print "Queue empty."
        while True:
            r = results.get(timeout=2)
            print "In Queue: %s." % str(r)
    except:
        pass

except KeyboardInterrupt:
    kvmp.cleanup()
    kvmp.terminate()

# TODO Trap SIGINT etc, kill child processes.

exit(0)

